/**
 * @file aspi.c
 * @author BananaJinn
 * @version $Id: aspi.c,v 1.14 2010/11/01 14:34:11 bananajinn Exp $
 * 円盤複写屋
 * Copyright (C) 2004-2010 BananaJinn<banana@mxh.mesh.ne.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/file.h>
#include <linux/cdrom.h>

#include "aspi.h"
#include "cmd.h"

/*#define DEBUG*/

int OpenAspi()
{
	return RET_OK;
}


void CloseAspi()
{
}

int InitializeDrive(DRIVE *drive)
{
	drive->hid = -1;
	drive->tid = -1;
	drive->fd = -1;

	return RET_OK;
}

void FreeDrive(DRIVE *drive)
{
	if(drive->fd > 0){
		close(drive->fd);
		drive->fd = -1;
	}
}

int SetAspiSetting(DRIVE *drive, int *ha, int *tg, DWORD *timeout)
{
	BOOL target_changed=FALSE;
	char *devname = NULL;

	if(ha!=NULL)
		if(*ha>=3)
			return RET_NG;
	if(tg!=NULL)
		if(*tg>=26)
			return RET_NG;

	if(ha!=NULL){
		drive->hid = *ha;
		target_changed=TRUE;
	}
	if(tg!=NULL){
		drive->tid = *tg;
		target_changed=TRUE;
	}
	if(timeout!=NULL){
		drive->timeout = *timeout;
	}

	if(target_changed){
		if(drive->fd>=0){
			close(drive->fd);
		}
	
		devname = GetDeviceNameByID(drive->hid, drive->tid);
#if defined(K24)
		drive->fd = open(devname, O_RDONLY|O_NONBLOCK);
#else
		drive->fd = open(devname, O_RDWR|O_NONBLOCK);
#endif
		if(drive->fd<0){
			/*printf("[%s] open error. errno=%d\n", devname, errno);*/
			free(devname);
			return RET_NG;
		}
		free(devname);
	}

	return RET_OK;
}

void GetAspiSetting(DRIVE *drive, int *ha, int *tg, DWORD *timeout)
{
	if(ha!=NULL){
		*ha = drive->hid;
	}
	if(tg!=NULL){
		*tg = drive->tid;
	}
	if(timeout!=NULL){
		*timeout = drive->timeout;
	}
}


#ifdef DEBUG
void dump_cgc(struct cdrom_generic_command *cgc)
{
	struct request_sense *sense = cgc->sense;
	int i;

	printf("data direction: %d\n", cgc->data_direction);
	printf("cdb: ");
	for (i = 0; i < 12; i++)
		printf("%02x ", cgc->cmd[i]);
	printf("\n");
  
	printf("buffer (%d): ", cgc->buflen);
	for (i = 0; i < cgc->buflen; i++)
		printf("%02x ", cgc->buffer[i]);
	printf("\n");
  
	if (!sense)
		return;
  
	printf("sense: %02x.%02x.%02x\n", sense->sense_key, sense->asc, sense->ascq);
}
#endif

int SendAspiCmd(DRIVE *drive, BYTE *cdb, BYTE *buf, DWORD buflen,
				BYTE *sensedata, DWORD senselen, BYTE reqflag)
{
	int ret;
	struct cdrom_generic_command cgc;
	struct request_sense sense_data;

	if(drive->fd<0){
		printf("Device not opened.\n");
		return RET_NG;
	}

	memset(&sense_data, 0, sizeof(struct request_sense));
	memset(&cgc, 0, sizeof(struct cdrom_generic_command));
	cgc.timeout = drive->timeout*100;
	cgc.buffer = buf;
	cgc.buflen = buflen;

	switch(reqflag){
	case REQ_DATAIN:
		cgc.data_direction = CGC_DATA_READ;
		break;
	case REQ_DATAOUT:
		cgc.data_direction = CGC_DATA_WRITE;
		break;
	default:
		cgc.data_direction = CGC_DATA_NONE;
	}

	cgc.sense = &sense_data;
	cgc.quiet = 1;
	memcpy(cgc.cmd, cdb, 12);

	ret = ioctl(drive->fd, CDROM_SEND_PACKET, &cgc);
	memcpy(sensedata, &sense_data,
		   senselen < sizeof(struct request_sense) ?
		   senselen : sizeof(struct request_sense));
#if 0
	if((drive->sense_data[2] & 0x0f) != 0x00 ||
	   drive->sense_data[12] != 0x00 ||
	   drive->sense_data[13] != 0x00)
		ret = RET_CMDERR;
#else
	if(ret!=0)
		ret = RET_CMDERR;
#endif

#ifdef DEBUG
	dump_cgc(&cgc);
#endif
	return ret;
}

int GetHostAdapterCount()
{
	return 2;
}

int GetMaxTarget()
{
	return 26;
}


/**
 * @brief デバイスファイル名からID番号を取得
 * @param[in] devfile デバイスファイル名(/dev/xxx または xxx)
 * @param[out] ha ホストアダプタID(/dev/hd?の場合は0、/dev/scd?の場合は1)
 * @param[out] tg ターゲットID(/dev/hdaを先頭とした番号または/dev/scd0を先頭とした番号)
 * @retval RET_OK 変換成功
 * @retval RET_NG 変換失敗(不正なデバイス名など)
 */
int GetDeviceIDByName(const char *devfile, int *ha, int *tg)
{
	if((ha == NULL) || (tg == NULL)){
		return RET_NG;
	}
	if(!strncmp(devfile, "/dev/", 5)){
		devfile += 5;
	}
	if(!strncmp(devfile, "hd", 2)){
		*ha = 0;
		*tg = devfile[2]-'a';
	}
	else if(!strncmp(devfile, "scd", 3)){
		*ha = 1;
		*tg = devfile[3]-'0';
	}
	else{
		return RET_NG;
	}
	return RET_OK;
}

/**
 * @brief ID番号からデバイスファイル名を取得
 * @param[in] ha ホストアダプタID(/dev/hd?の場合は0、/dev/scd?の場合は1)
 * @param[in] tg ターゲットID(/dev/hdaを先頭とした番号または/dev/scd0を先頭とした番号)
 * @return 変換成功時はデバイスファイル名を、エラー時はNULLを返す。返されたバッファは free
 *         でメモリ解放が必要。
 */
char *GetDeviceNameByID(int ha, int tg)
{
	char *buf = NULL;

	if(ha == 0){
		buf = malloc(8+1);
		snprintf(buf, 8+1, "/dev/hd%c", tg+'a');
	}
	else{
		buf = malloc(9+1);
		snprintf(buf, 9+1, "/dev/scd%c", tg+'0');
	}
	return buf;
}
